@@ -32,6 +32,7 @@
 #include <gst/gst.h>
 #include <gst/tag/tag.h>
 #include <profiles/gnome-media-profiles.h>
+#include "sound-juicer.h"
 #include "sj-extractor.h"
 #include "sj-structures.h"
 #include "sj-error.h"
@@ -58,7 +59,6 @@ static guint signals[LAST_SIGNAL] = { 0 };
 #define DEFAULT_AUDIO_PROFILE_NAME "cdlossy"
 
 /* Element names */
-#define CD_SRC "cdparanoiasrc"
 #define FILE_SINK "gnomevfssink"
 
 struct SjExtractorPrivate {
@@ -67,7 +67,7 @@ struct SjExtractorPrivate {
   /** If the pipeline needs to be re-created */
   gboolean rebuild_pipeline;
   /* The gstreamer pipeline elements */
-  GstElement *pipeline, *cdparanoia, *queue, *thread, *encoder, *filesink;
+  GstElement *pipeline, *cdsrc, *queue, *thread, *encoder, *filesink;
   GstFormat track_format;
   char *device_path;
   int paranoia_mode;
@@ -150,6 +150,7 @@ static void sj_extractor_set_property (GObject *object, guint property_id,
   case PROP_PROFILE:
     priv->profile = g_value_get_object (value);
     priv->rebuild_pipeline = TRUE;
+    g_object_notify (object, "profile");
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -209,76 +210,6 @@ static void eos_cb (GstBus *bus, GstMessage *message, gpointer user_data)
   g_signal_emit (extractor, signals[COMPLETION], 0);
 }
 
-/* Stolen from gst-plugins-good/ext/gconf/gconf.c */
-static GstPad *
-my_gst_bin_find_unconnected_pad (GstBin * bin, GstPadDirection direction)
-{
-  GstPad *pad = NULL;
-  GList *elements = NULL;
-  const GList *pads = NULL;
-  GstElement *element = NULL;
-
-  GST_OBJECT_LOCK (bin);
-  elements = bin->children;
-  /* traverse all elements looking for unconnected pads */
-  while (elements && pad == NULL) {
-    element = GST_ELEMENT (elements->data);
-    GST_OBJECT_LOCK (element);
-    pads = element->pads;
-    while (pads) {
-      GstPad *testpad = GST_PAD (pads->data);
-
-      /* check if the direction matches */
-      if (GST_PAD_DIRECTION (testpad) == direction) {
-        GST_OBJECT_LOCK (testpad);
-        if (GST_PAD_PEER (testpad) == NULL) {
-          GST_OBJECT_UNLOCK (testpad);
-          /* found it ! */
-          pad = testpad;
-          break;
-        }
-        GST_OBJECT_UNLOCK (testpad);
-      }
-      pads = g_list_next (pads);
-    }
-    GST_OBJECT_UNLOCK (element);
-    elements = g_list_next (elements);
-  }
-  GST_OBJECT_UNLOCK (bin);
-
-  return pad;
-}
-
-/* Stolen from gst-plugins-good/ext/gconf/gconf.c */
-static GstElement *
-my_gst_gconf_render_bin_from_description (const gchar * description)
-{
-  GstElement *bin = NULL;
-  GstPad *pad = NULL;
-  GError *error = NULL;
-  gchar *desc = NULL;
-
-  /* parse the pipeline to a bin */
-  desc = g_strdup_printf ("bin.( %s )", description);
-  bin = GST_ELEMENT (gst_parse_launch (desc, &error));
-  g_free (desc);
-  if (error) {
-    GST_ERROR ("gstgconf: error parsing pipeline %s\n%s\n",
-        description, error->message);
-    g_error_free (error);
-    return NULL;
-  }
-
-  /* find pads and ghost them if necessary */
-  if ((pad = my_gst_bin_find_unconnected_pad (GST_BIN (bin), GST_PAD_SRC))) {
-    gst_element_add_pad (bin, gst_ghost_pad_new ("src", pad));
-  }
-  if ((pad = my_gst_bin_find_unconnected_pad (GST_BIN (bin), GST_PAD_SINK))) {
-    gst_element_add_pad (bin, gst_ghost_pad_new ("sink", pad));
-  }
-  return bin;
-}
-
 static GstElement* build_encoder (SjExtractor *extractor)
 {
   SjExtractorPrivate *priv;
@@ -290,7 +221,7 @@ static GstElement* build_encoder (SjExtractor *extractor)
   g_return_val_if_fail (priv->profile != NULL, NULL);
  
   pipeline = g_strdup_printf ("audioconvert ! %s", gm_audio_profile_get_pipeline (priv->profile));
-  element = my_gst_gconf_render_bin_from_description (pipeline);
+  element = gst_parse_bin_from_description (pipeline, TRUE, NULL); /* TODO: return error */
   g_free(pipeline);
   return element;
 }
@@ -304,6 +235,9 @@ static void error_cb (GstBus *bus, GstMessage *message, gpointer user_data)
   gst_element_set_state (extractor->priv->pipeline, GST_STATE_NULL);
   extractor->priv->rebuild_pipeline = TRUE;
 
+  g_source_remove (extractor->priv->tick_id);
+  extractor->priv->tick_id = 0;
+
   gst_message_parse_error (message, &error, NULL);
   g_signal_emit (extractor, signals[ERROR], 0, error);
   g_error_free (error);
@@ -338,16 +272,18 @@ static void build_pipeline (SjExtractor *extractor)
   g_signal_connect (G_OBJECT (bus), "message::error", G_CALLBACK (error_cb), extractor);
 
   /* Read from CD */
-  priv->cdparanoia = gst_element_factory_make (CD_SRC, "cd_src");
-  if (priv->cdparanoia == NULL) {
+  priv->cdsrc = gst_element_factory_make (CD_SRC, "cd_src");
+  if (priv->cdsrc == NULL) {
     g_set_error (&priv->construct_error,
                  SJ_ERROR, SJ_ERROR_INTERNAL_ERROR,
-                 _("Could not create GStreamer cdparanoia reader"));
+                 _("Could not create GStreamer CD reader"));
     return;
   }
 
-  g_object_set (G_OBJECT (priv->cdparanoia), "device", priv->device_path, NULL);
-  g_object_set (G_OBJECT (priv->cdparanoia), "paranoia-mode", priv->paranoia_mode, NULL);
+  g_object_set (G_OBJECT (priv->cdsrc), "device", priv->device_path, NULL);
+  g_object_set (G_OBJECT (priv->cdsrc), "paranoia-mode", priv->paranoia_mode, NULL);
+  /* Need to do this, as playback will have locked the read speed to 2x previously */
+  g_object_set (G_OBJECT (priv->cdsrc), "read-speed", G_MAXINT, NULL);
 
   /* Get the track format for seeking later */
   priv->track_format = gst_format_get_by_nick ("track");
@@ -379,13 +315,13 @@ static void build_pipeline (SjExtractor *extractor)
   g_signal_connect (G_OBJECT (priv->filesink), "allow-overwrite", G_CALLBACK (just_say_yes), extractor);
 
   /* Add the elements to the pipeline */
-  gst_bin_add_many (GST_BIN (priv->pipeline), priv->cdparanoia, priv->queue, priv->encoder, priv->filesink, NULL);
+  gst_bin_add_many (GST_BIN (priv->pipeline), priv->cdsrc, priv->queue, priv->encoder, priv->filesink, NULL);
 
   /* Link it all together */
-  if (!gst_element_link_many (priv->cdparanoia, priv->queue, priv->encoder, priv->filesink, NULL)) {
-    /* TODO: need to produce a GError here */
-    g_warning ("Cannot link pipeline, very bad!");
-    g_object_unref (priv->pipeline);
+  if (!gst_element_link_many (priv->cdsrc, priv->queue, priv->encoder, priv->filesink, NULL)) {
+    g_set_error (&priv->construct_error,
+                 SJ_ERROR, SJ_ERROR_INTERNAL_ERROR,
+                 _("Could not link pipeline"));
     return;
   }
 
@@ -396,17 +332,18 @@ static gboolean tick_timeout_cb(SjExtractor *extractor)
 {
   gint64 nanos;
   gint secs;
-  GstState state;
+  GstState state, pending_state;
   static GstFormat format = GST_FORMAT_TIME;
 
   g_return_val_if_fail (SJ_IS_EXTRACTOR (extractor), FALSE);
 
-  gst_element_get_state (extractor->priv->pipeline, &state, NULL, GST_CLOCK_TIME_NONE);
-  if (state != GST_STATE_PLAYING) {
+  gst_element_get_state (extractor->priv->pipeline, &state, &pending_state, 0);
+  if (state != GST_STATE_PLAYING && pending_state != GST_STATE_PLAYING) {
+    extractor->priv->tick_id = 0;
     return FALSE;
   }
 
-  if (!gst_element_query_position (extractor->priv->cdparanoia, &format, &nanos)) {
+  if (!gst_element_query_position (extractor->priv->cdsrc, &format, &nanos)) {
     g_warning (_("Could not get current track position"));
     return TRUE;
   }
@@ -448,8 +385,8 @@ void sj_extractor_set_device (SjExtractor *extractor, const char* device)
   g_free (extractor->priv->device_path);
   extractor->priv->device_path = g_strdup (device);
 
-  if (extractor->priv->cdparanoia != NULL)
-    g_object_set (G_OBJECT (extractor->priv->cdparanoia), "device", device, NULL);
+  if (extractor->priv->cdsrc != NULL)
+    g_object_set (G_OBJECT (extractor->priv->cdsrc), "device", device, NULL);
 }
 
 void sj_extractor_set_paranoia (SjExtractor *extractor, const int paranoia_mode)
@@ -457,12 +394,13 @@ void sj_extractor_set_paranoia (SjExtractor *extractor, const int paranoia_mode)
   g_return_if_fail (SJ_IS_EXTRACTOR (extractor));
 
   extractor->priv->paranoia_mode = paranoia_mode;
-  if (extractor->priv->cdparanoia != NULL)
-    g_object_set (G_OBJECT (extractor->priv->cdparanoia), "paranoia-mode", paranoia_mode, NULL);
+  if (extractor->priv->cdsrc != NULL)
+    g_object_set (G_OBJECT (extractor->priv->cdsrc), "paranoia-mode", paranoia_mode, NULL);
 }
 
 void sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, const char* url, GError **error)
 {
+  GstStateChangeReturn state_ret;
   SjExtractorPrivate *priv;
   static GstFormat format = GST_FORMAT_TIME;
   gint64 nanos;
@@ -481,8 +419,7 @@ void sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *tra
   if (priv->rebuild_pipeline != FALSE) {
     build_pipeline (extractor);
     if (priv->construct_error != NULL) {
-      *error = g_error_copy (priv->construct_error);
-      g_error_free (priv->construct_error);
+      g_propagate_error (error, priv->construct_error);
       priv->construct_error = NULL;
       return;
     }
@@ -506,8 +443,7 @@ void sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *tra
                                GST_TAG_TRACK_NUMBER, track->number,
                                GST_TAG_TRACK_COUNT, track->album->number,
                                GST_TAG_ALBUM, track->album->title,
-                               GST_TAG_ENCODER, _("Sound Juicer"),
-                               GST_TAG_ENCODER_VERSION, VERSION,
+                               GST_TAG_DURATION, track->duration * GST_SECOND,
                                NULL);
 
 #ifdef GST_TAG_MUSICBRAINZ_ALBUMID
@@ -559,6 +495,7 @@ void sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *tra
       break;
     case GST_ITERATOR_RESYNC:
       // TODO?
+      g_warning ("Got GST_ITERATOR_RESYNC, not sure what to do");
       gst_iterator_resync (iter);
       break;
     case GST_ITERATOR_ERROR:
@@ -572,13 +509,32 @@ void sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *tra
   gst_iterator_free (iter);
 
   /* Seek to the right track */
-  g_object_set (G_OBJECT (priv->cdparanoia), "track", track->number, NULL);
+  g_object_set (G_OBJECT (priv->cdsrc), "track", track->number, NULL);
   
   /* Let's get ready to rumble! */
   gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
-  gst_element_get_state (priv->pipeline, NULL, NULL, GST_CLOCK_TIME_NONE);
 
-  if (!gst_element_query_position (priv->cdparanoia, &format, &nanos)) {
+  /* Wait for state change to either complete or fail */
+  state_ret = gst_element_get_state (priv->pipeline, NULL, NULL, GST_CLOCK_TIME_NONE);
+
+  if (state_ret == GST_STATE_CHANGE_FAILURE) {
+    GstMessage *msg;
+
+    msg = gst_bus_poll (GST_ELEMENT_BUS (priv->pipeline), GST_MESSAGE_ERROR, 0);
+    if (msg) {
+      gst_message_parse_error (msg, error, NULL);
+      gst_message_unref (msg);
+    } else if (error) {
+      /* this should never happen, create generic error just in case */
+      *error = g_error_new (SJ_ERROR, SJ_ERROR_INTERNAL_ERROR,
+                            "Error starting ripping pipeline");
+    }
+    gst_element_set_state (priv->pipeline, GST_STATE_NULL);
+    priv->rebuild_pipeline = TRUE;
+    return;
+  }
+
+  if (!gst_element_query_position (priv->cdsrc, &format, &nanos)) {
     g_warning (_("Could not get track start position"));
     return;
   }
