@@ -37,7 +37,6 @@ extern GladeXML *glade;
 extern GtkWidget *main_window;
 
 static GtkWidget *audio_profile;
-static GtkWidget *prefs_dialog;
 static GtkWidget *cd_option, *path_option, *file_option, *basepath_fcb, *check_strip, *check_eject;
 static GtkWidget *path_example_label;
 
@@ -88,7 +87,7 @@ void prefs_profile_changed (GtkWidget *widget, gpointer user_data)
 /**
  * Show the gnome help browser
  */
-void show_help ()
+void show_help (GtkWindow *parent)
 {
   GError *error = NULL;
 
@@ -96,7 +95,7 @@ void show_help ()
   if (error) {
     GtkWidget *dialog;
 
-    dialog = gtk_message_dialog_new (GTK_WINDOW (prefs_dialog),
+    dialog = gtk_message_dialog_new (parent,
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
                                      GTK_MESSAGE_ERROR,
                                      GTK_BUTTONS_CLOSE,
@@ -201,12 +200,9 @@ static void pattern_label_update (void)
   GMAudioProfile *profile;
 
   static AlbumDetails sample_album = {
-    /* Translators: this is the title of a common album. You don't need to change it */
-    N_("Help!"), /* title */
-    /* Translators: this is the artist of a common album. You don't need to change it */
-    N_("The Beatles"), /* artist */
-    /* Translators: this is the artist sortname of a common album. You don't need to change it */
-    N_("Beatles, The"), /* sortname */
+    "Help!", /* title */
+    "The Beatles", /* artist */
+    "Beatles, The", /* sortname */
     NULL, /* genre */
     0, /* number of tracks*/
     NULL, /* track list */
@@ -217,31 +213,21 @@ static void pattern_label_update (void)
   static TrackDetails sample_track = {
     &sample_album,  /*album */
     7, /* track number */
-    /* Translators: this is the title of a common song. You don't need to change it */
-    N_("Ticket To Ride"), /* title */
-    /* Translators: this is the artist of a common song. You don't need to change it */
-    N_("The Beatles"), /* artist */
-    /* Translators: this is the artist sortname of a common song. You don't need to change it */
-    N_("Beatles, The"), /* sortname */
+    "Ticket To Ride", /* title */
+    "The Beatles", /* artist */
+    "Beatles, The", /* sortname */
     0, /* duration */
     NULL, /* track ID */
     NULL, /* artist ID */
     {0,0,0,0}, /* treeview iterator */
   };
-  /* Magic to i18n-ize the sample strings. */
-  static gboolean been_here = FALSE;
-  if (!been_here) {
-    sample_album.title = _(sample_album.title);
-    sample_album.artist = _(sample_album.artist);
-    sample_album.artist_sortname = _(sample_album.artist_sortname);
-    sample_track.title = _(sample_track.title);
-    sample_track.artist = _(sample_track.artist);
-    sample_track.artist_sortname = _(sample_track.artist_sortname);
-    been_here = TRUE;
-  }
   
   g_object_get (extractor, "profile", &profile, NULL);
-  
+  /* It's possible the profile isn't set, in which case do nothing */
+  if (!profile) {
+    return;
+  }
+
   /* TODO: sucky. Replace with get-gconf-key-with-default mojo */
   file_pattern = gconf_client_get_string (gconf_client, GCONF_FILE_PATTERN, NULL);
   if (file_pattern == NULL) {
@@ -264,7 +250,7 @@ static void pattern_label_update (void)
   format = g_strconcat ("<small><i><b>Example Path:</b> ",
                         example,
                         ".",
-                        gm_audio_profile_get_extension(profile),
+                        gm_audio_profile_get_extension (profile),
                         "</i></small>", NULL);
   g_free (example);
   
@@ -330,21 +316,35 @@ static void populate_pattern_combo (GtkComboBox *combo, const FilePattern *patte
   gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
 }
 
+static void
+on_response (GtkDialog *dialog, gint response, gpointer user_data)
+{
+  if (response == GTK_RESPONSE_HELP) {
+    show_help (GTK_WINDOW (dialog));
+  } else {
+    gtk_widget_hide (GTK_WIDGET (dialog));
+  }
+}
+
 /**
  * Clicked on Preferences in the UI
  */
 void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
 {
-  int rc;
+  static GtkWidget *prefs_dialog = NULL;
 
-  if (prefs_dialog == NULL) {
-    const char * labels[] = { "cd_label", "path_label", "folder_label", "file_label", "profile_label" };
+  if (prefs_dialog) {
+    gtk_window_present (GTK_WINDOW (prefs_dialog));
+  } else {
+    const char *labels[] = { "cd_label", "path_label", "folder_label", "file_label", "profile_label" };
     guint i;
     GtkSizeGroup *group;
     GConfBridge *bridge = gconf_bridge_get ();
 
     prefs_dialog = glade_xml_get_widget (glade, "prefs_dialog");
     g_assert (prefs_dialog != NULL);
+    g_object_add_weak_pointer (G_OBJECT (prefs_dialog), (gpointer*)&prefs_dialog);
+
     gtk_window_set_transient_for (GTK_WINDOW (prefs_dialog), GTK_WINDOW (main_window));
 
     group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
@@ -383,22 +383,14 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
     gconf_client_notify_add (gconf_client, GCONF_FILE_PATTERN, file_pattern_changed_cb, NULL, NULL, NULL);
 
     g_signal_connect (extractor, "notify::profile", pattern_label_update, NULL);
+
+    baseuri_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEURI, NULL, TRUE, NULL), NULL);
+    audio_profile_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_PROFILE, NULL, TRUE, NULL), NULL);
+    file_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_FILE_PATTERN, NULL, TRUE, NULL), NULL);
+    path_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PATH_PATTERN, NULL, TRUE, NULL), NULL);
+
+    g_signal_connect (GTK_DIALOG (prefs_dialog), "response", G_CALLBACK (on_response), NULL);
+
+    gtk_widget_show_all (prefs_dialog);
   }
-  baseuri_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEURI, NULL, TRUE, NULL), NULL);
-  audio_profile_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_PROFILE, NULL, TRUE, NULL), NULL);
-  file_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_FILE_PATTERN, NULL, TRUE, NULL), NULL);
-  path_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PATH_PATTERN, NULL, TRUE, NULL), NULL);
-
-  gtk_widget_show_all (prefs_dialog);
-
-  while (1) {
-      rc = gtk_dialog_run (GTK_DIALOG (prefs_dialog));
-
-      if (rc == GTK_RESPONSE_HELP)
-	  show_help();
-      else if (rc == GTK_RESPONSE_CLOSE || rc == GTK_RESPONSE_DELETE_EVENT)
-	  break;
-      }
-      
-  gtk_widget_hide (prefs_dialog);
 }
